package com.ruoyi.project.plant.service.impl;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.plant.mapper.TTargetmeasuresMapper;
import com.ruoyi.project.plant.domain.TTargetmeasures;
import com.ruoyi.project.plant.service.ITTargetmeasuresService;

import javax.annotation.Resource;

/**
 * 目标措施Service业务层处理
 *
 * @author ruoyi
 * @date 2021-07-05
 */
@Service
public class TTargetmeasuresServiceImpl implements ITTargetmeasuresService
{
    @Resource
    private TTargetmeasuresMapper tTargetmeasuresMapper;

    /**
     * 查询目标措施
     *
     * @param id 目标措施ID
     * @return 目标措施
     */
    @Override
    public TTargetmeasures selectTTargetmeasuresById(Long id)
    {
        return tTargetmeasuresMapper.selectTTargetmeasuresById(id);
    }

    /**
     * 查询目标措施
     *
     * @param targetlistId 目标列表ID
     * @return 目标措施
     */
    @Override
    public TTargetmeasures selectTTargetmeasuresByListId(Long targetlistId) {
        return tTargetmeasuresMapper.selectTTargetmeasuresByListId(targetlistId);
    }

    /**
     * 查询目标措施列表
     *
     * @param tTargetmeasures 目标措施
     * @return 目标措施
     */
    @Override
    public List<TTargetmeasures> selectTTargetmeasuresList(TTargetmeasures tTargetmeasures)
    {
        return tTargetmeasuresMapper.selectTTargetmeasuresList(tTargetmeasures);
    }

    /**
     * 新增目标措施
     *
     * @param tTargetmeasures 目标措施
     * @return 结果
     */
    @Override
    public int insertTTargetmeasures(TTargetmeasures tTargetmeasures)
    {
        return tTargetmeasuresMapper.insertTTargetmeasures(tTargetmeasures);
    }

    /**
     * 修改目标措施
     *
     * @param tTargetmeasures 目标措施
     * @return 结果
     */
    @Override
    public int updateTTargetmeasures(TTargetmeasures tTargetmeasures)
    {
        return tTargetmeasuresMapper.updateTTargetmeasures(tTargetmeasures);
    }

    /**
     * 批量删除目标措施
     *
     * @param ids 需要删除的目标措施ID
     * @return 结果
     */
    @Override
    public int deleteTTargetmeasuresByIds(Long[] ids)
    {
        return tTargetmeasuresMapper.deleteTTargetmeasuresByIds(ids);
    }

    /**
     * 删除目标措施信息
     *
     * @param id 目标措施ID
     * @return 结果
     */
    @Override
    public int deleteTTargetmeasuresById(Long id)
    {
        return tTargetmeasuresMapper.deleteTTargetmeasuresById(id);
    }
}
